@@ -129,13 +129,18 @@ get_id_string (GtkWidget *widget)
 static gboolean
 print_id (GtkWidget *widget)
 {
+        char *id;
+
         profile_start ("start", NULL);
 
-        printf ("WINDOW ID=%s\n", get_id_string (widget));
+        id = get_id_string (widget);
+        printf ("WINDOW ID=%s\n", id);
         fflush (stdout);
 
         profile_end ("end", NULL);
 
+        g_free (id);
+
         return FALSE;
 }
 
@@ -215,7 +220,7 @@ popup_dialog_idle (void)
  * Copyright (c) 2005 William Jon McCann <mccann@jhu.edu>
  *
  * Initializations that potentially take place as a priveleged user:
-   If the xscreensaver executable is setuid root, then these initializations
+   If the executable is setuid root, then these initializations
    are run as root, before discarding privileges.
 */
 static gboolean
